"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnhancedState = void 0;
const internal_1 = require("../internal");
class EnhancedState extends internal_1.State {
    constructor(agileInstance, initialValue, config = {}) {
        super(agileInstance, initialValue, config);
        this.isPersisted = false;
        this.computeExistsMethod = (v) => {
            return v != null;
        };
    }
    setKey(value) {
        var _a, _b;
        const oldKey = this._key;
        super.setKey(value);
        if (value != null && ((_a = this.persistent) === null || _a === void 0 ? void 0 : _a._key) === oldKey)
            (_b = this.persistent) === null || _b === void 0 ? void 0 : _b.setKey(value);
        return this;
    }
    undo(config = {}) {
        this.set(this.previousStateValue, config);
        return this;
    }
    reset(config = {}) {
        this.set(this.initialStateValue, config);
        return this;
    }
    patch(targetWithChanges, config = {}) {
        config = internal_1.defineConfig(config, {
            addNewProperties: true,
        });
        if (!internal_1.isValidObject(this.nextStateValue, true)) {
            internal_1.LogCodeManager.log('14:03:02');
            return this;
        }
        if (!internal_1.isValidObject(targetWithChanges, true)) {
            internal_1.LogCodeManager.log('00:03:01', ['TargetWithChanges', 'object']);
            return this;
        }
        if (Array.isArray(targetWithChanges) &&
            Array.isArray(this.nextStateValue)) {
            this.nextStateValue = [
                ...this.nextStateValue,
                ...targetWithChanges,
            ];
        }
        else {
            this.nextStateValue = internal_1.flatMerge(this.nextStateValue, targetWithChanges, { addNewProperties: config.addNewProperties });
        }
        this.ingest(internal_1.removeProperties(config, ['addNewProperties']));
        return this;
    }
    watch(keyOrCallback, callback) {
        const generateKey = internal_1.isFunction(keyOrCallback);
        let _callback;
        let key;
        if (generateKey) {
            key = internal_1.generateId();
            _callback = keyOrCallback;
        }
        else {
            key = keyOrCallback;
            _callback = callback;
        }
        if (!internal_1.isFunction(_callback)) {
            internal_1.LogCodeManager.log('00:03:01', ['Watcher Callback', 'function']);
            return this;
        }
        this.addSideEffect(key, (instance) => {
            _callback(instance.value, key);
        }, { weight: 0 });
        return generateKey ? key : this;
    }
    removeWatcher(key) {
        this.removeSideEffect(key);
        return this;
    }
    onInaugurated(callback) {
        const watcherKey = 'InauguratedWatcherKey';
        this.watch(watcherKey, (value, key) => {
            callback(value, key);
            this.removeSideEffect(watcherKey);
        });
        return this;
    }
    interval(handler, delay) {
        if (!internal_1.isFunction(handler)) {
            internal_1.LogCodeManager.log('00:03:01', ['Interval Callback', 'function']);
            return this;
        }
        if (this.currentInterval) {
            internal_1.LogCodeManager.log('14:03:03', [], this.currentInterval);
            return this;
        }
        this.currentInterval = setInterval(() => {
            this.set(handler(this._value));
        }, delay !== null && delay !== void 0 ? delay : 1000);
        return this;
    }
    clearInterval() {
        if (this.currentInterval) {
            clearInterval(this.currentInterval);
            delete this.currentInterval;
        }
    }
    get exists() {
        return !this.isPlaceholder && this.computeExistsMethod(this.value);
    }
    computeExists(method) {
        if (!internal_1.isFunction(method)) {
            internal_1.LogCodeManager.log('00:03:01', ['Compute Exists Method', 'function']);
            return this;
        }
        this.computeExistsMethod = method;
        return this;
    }
    computeValue(method) {
        if (!internal_1.isFunction(method)) {
            internal_1.LogCodeManager.log('00:03:01', ['Compute Value Method', 'function']);
            return this;
        }
        this.computeValueMethod = method;
        this.set(this.nextStateValue);
        return this;
    }
    is(value) {
        return internal_1.equal(value, this.value);
    }
    isNot(value) {
        return internal_1.notEqual(value, this.value);
    }
    invert() {
        switch (typeof this.nextStateValue) {
            case 'boolean':
                this.set(!this.nextStateValue);
                break;
            case 'object':
                if (Array.isArray(this.nextStateValue))
                    this.set(this.nextStateValue.reverse());
                break;
            case 'string':
                this.set(this.nextStateValue.split('').reverse().join(''));
                break;
            case 'number':
                this.set((this.nextStateValue * -1));
                break;
            default:
                internal_1.LogCodeManager.log('14:03:04', [typeof this.nextStateValue]);
        }
        return this;
    }
    persist(keyOrConfig = {}, config = {}) {
        let _config;
        let key;
        if (internal_1.isValidObject(keyOrConfig)) {
            _config = keyOrConfig;
            key = this._key;
        }
        else {
            _config = config || {};
            key = keyOrConfig;
        }
        _config = internal_1.defineConfig(_config, {
            loadValue: true,
            storageKeys: [],
            defaultStorageKey: null,
        });
        if (this.persistent != null && this.isPersisted)
            return this;
        this.persistent = new internal_1.StatePersistent(this, {
            loadValue: _config.loadValue,
            storageKeys: _config.storageKeys,
            key: key,
            defaultStorageKey: _config.defaultStorageKey,
        });
        return this;
    }
    onLoad(callback) {
        if (!this.persistent)
            return this;
        if (!internal_1.isFunction(callback)) {
            internal_1.LogCodeManager.log('00:03:01', ['OnLoad Callback', 'function']);
            return this;
        }
        this.persistent.onLoad = callback;
        if (this.isPersisted)
            callback(true);
        return this;
    }
    getPersistableValue() {
        return this._value;
    }
}
exports.EnhancedState = EnhancedState;
